import os
from datetime import datetime

import requests

from tba_py.cache import JsonCacher

__author__ = "Kestin Goforth"
__email__ = "kgoforth1503@gmail.com"
__url__ = "http://github.com/kforth/tba_py/"
__version__ = '1.0.6'
dir_path = os.path.dirname(os.path.realpath(__file__))

"""

## BlueAllianceAPI
Functions names mimic The Blue Alliance API endpoint urls such that
GET 'http://thebluealliance.com/api/v2/event/2016onwa/rankings'
is represented by
BlueAllianceAPI.get_event_rankings('2016onwa')

"""


class BlueAllianceAPI(object):
    tba_url = "http://thebluealliance.com/api/v2"

    """
        A Python wrapper for The Blue Alliance's public API.
        user_id           -- Your personal name, team name, company name, etc.
        app_description   -- The name of your app or what it does.
        app_version       -- The current version number of your app.
        enable_caching    -- Whether or not to enable the internal cacher.
        cache_db_path     -- The file to store the cache database in, if enabled.
        cache_refresh_age -- The number of hours to wait before allowing an
                             entry to be refreshed, if caching is enabled.
    """

    def __init__(self, user_id, app_description, app_version,
                 enable_caching=False, cache_db_path=None, cache_refresh_age=1, prefer_cached=True):
        self.user_id = user_id.replace(":", "-")
        self.app_description = app_description.replace(":", "-")
        self.app_version = app_version.replace(":", "-")
        self.headers = {'X-TBA-App-Id': self.user_id + ":" + self.app_description + ":v" + self.app_version}
        self.caching_enabled = enable_caching
        self.prefer_cached = prefer_cached

        if self.caching_enabled:
            if cache_db_path is None:
                cache_db_path = dir_path + "/db.json"
            self._setup_caching(cache_db_path, cache_refresh_age)

    # Teams
    def get_teams(self, page_num):
        return self._get("/teams/" + page_num)

    def get_team(self, team, suffix=None):
        if team[0:3] == 'frc':
            team = team[3:]
        return self._get("/team/frc" + team + (suffix if suffix is not None else ""))

    def get_team_events(self, team, year=datetime.now().year):
        return self.get_team(team, "/" + year + "/events")

    def get_team_event_awards(self, team, event_id):
        return self.get_team(team, "/event/" + event_id + "/awards")

    def get_team_event_matches(self, team, event_id):
        return self.get_team(team, "/event/" + event_id + "/matches")

    def get_team_years_participated(self, team):
        return self.get_team(team, "/years_participated")

    def get_team_media(self, team, year=None):
        return self.get_team(team, (("/" + year) if year is not None else "") + "/media")

    def get_team_history_events(self, team):
        return self.get_team(team, "/history/events")

    def get_team_history_awards(self, team):
        return self.get_team(team, "/history/awards")

    def get_team_history_robots(self, team):
        return self.get_team(team, "/history/robots")

    def get_team_history_districts(self, team):
        return self.get_team(team, "/history/districts")

    # Events
    def get_events(self, year=datetime.now().year):
        return self._get("/events/" + year)

    def get_event(self, event_id, suffix=None):
        return self._get("/event/" + event_id + (suffix if suffix is not None else ""))

    def get_event_teams(self, event_id):
        return self.get_event(event_id, "/teams")

    def get_event_matches(self, event_id):
        return self.get_event(event_id, "/matches")

    def get_event_stats(self, event_id):
        return self.get_event(event_id, "/stats")

    def get_event_rankings(self, event_id):
        return self.get_event(event_id, "/rankings")

    def get_event_awards(self, event_id):
        return self.get_event(event_id, "/awards")

    def get_event_district_points(self, event_id):
        return self.get_event(event_id, "/district_points")

    # Match
    """ Example match_key: '2014cmp_f1m1' """

    def get_single_match(self, match_key):
        return self._get("/match/" + match_key)

    # District
    def get_districts(self, year=datetime.now().year):
        return self._get("/districts/" + year)

    def _get_district(self, district_id, year, suffix):
        return self._get("/districts/" + district_id + "/" + year + suffix)

    def get_district_events(self, district_id, year):
        return self._get_district(district_id, year, "/events")

    def get_district_rankings(self, district_id, year):
        return self._get_district(district_id, year, "/rankings")

    def get_district_teams(self, district_id, year):
        return self._get_district(district_id, year, "/teams")

    def _get(self, url):
        if self.caching_enabled and self.prefer_cached:
            cached = self.cacher.access(url)
            if cached is not None and self.cacher.cache_valid(url):
                return cached
        try:
            result = requests.get(self.tba_url + url, headers=self.headers)
            if result.status_code == 200:
                if self.caching_enabled:
                    self.cacher.offer(url, result.json())
                return result.json()
            else:
                raise Exception()
        except Exception as ex:
            if self.caching_enabled:
                cached = self.cacher.access(url)
                return cached
            else:
                raise ex

    def _setup_caching(self, fpath, age):
        self.cacher = JsonCacher(fpath, age, self.tba_url)
